from glob import glob
import numpy as np
import pandas as pd
from relation_matrix import relation_matrix
from partition_accuracy import partition_accuracy

def colony_bestfamily(folder, partition):
    """
    Assess the accuracy of sibship assignments for a set of Colony analyses based
    on the .BestFSFamily files and a reference partition.
    
    Parameters
    ----------
    folder: str
        Path to the folder where file are to be found.
    partition: array
        Reference partition structure to refer to. This should be a list or
        vector of the same length as the number of offspring.
    
    Returns
    -------
    DataFrame listing number of loci, replicate number, accuracy of sibship
    reconstruction overall, for true full sibship relationships, true half-sib
    relationships, and the number of families inferred.
    """
    # filenames containing ML partitions
    filenames = glob(folder + '*.BestFSFamily')
    # dataframe to store information about the Colony analyses.
    full_acc = np.zeros([len(filenames), 6])
    # Number of dads and replicate number for each dataset.
    full_acc[:,0] = [filenames[f].split('_l')[1].split('_r')[0] for f in range(len(filenames))] 
    full_acc[:,1] = [filenames[f].split('_r')[1].split('_i')[0] for f in range(len(filenames))] 

    partitions = [] # empty list to store ML partitions.
    # loop across each file and extract partition structure.
    for filename in filenames:
        with open(filename) as f:
            data = f.readlines() # open file
            # extract families
            l = []
            for line in data[1:]:
                words = line.split()          # split each line into columns.
                l = l + [words[3].split(',')] # split the list of offspring names.
            # Label family groups
            y = []
            for i in range(len(l)):
                y = y + [i for x in range(len(l[i]))] # label individuals in each sibship by a common label.
            partitions = partitions + [y] # append to partitions

    # Accuracies of relationship reconstructions
    for p in range(len(partitions)):
        full_acc[p,2] = partition_accuracy(partition, partitions[p], 'all')
        full_acc[p,3] = partition_accuracy(partition, partitions[p], 'fs')
        full_acc[p,4] = partition_accuracy(partition, partitions[p], 'hs')
        full_acc[p,5] = len(np.unique(partitions[p]))
    
    header=['nloci','rep','acc_all','acc_fs','acc_hs','nfamilies']
    full_acc = pd.DataFrame(full_acc, columns=header)
    return full_acc